#include <errno.h>

#include "FuseThread.h"

struct fuse_operations FuseThread::ops;

const char *FuseThread::hello_str = "Hello World!\n";
const char *FuseThread::hello_path = "/hello";

FuseThread::FuseThread(QObject *parent) :
        QThread(parent)
{
	ops.getattr	= FuseThread::getattr;
	ops.readdir	= FuseThread::readdir;
	ops.open	= FuseThread::open;
	ops.read	= FuseThread::read;
}

void FuseThread::run()
{
	struct fuse_args args = FUSE_ARGS_INIT(0, NULL);
	struct fuse_chan *ch;
	char *mountpoint = "/home/aither/.HaveClip/files";

	if( (ch = fuse_mount(mountpoint, &args)) != NULL )
	{
		struct fuse *fuse = fuse_new(ch, &args, &ops, sizeof(ops), NULL);

		if(fuse != NULL)
		{
			struct fuse_session *se = fuse_get_session(fuse);

			if(fuse_set_signal_handlers(se) != -1)
				fuse_loop(fuse);

			fuse_teardown(fuse, mountpoint);
		}

		fuse_unmount(mountpoint, ch);
	}
}

int FuseThread::getattr(const char *path, struct stat *stbuf)
{
	int res = 0;

	memset(stbuf, 0, sizeof(struct stat));
	if (strcmp(path, "/") == 0) {
		stbuf->st_mode = S_IFDIR | 0755;
		stbuf->st_nlink = 2;
	} else if (strcmp(path, hello_path) == 0) {
		stbuf->st_mode = S_IFREG | 0444;
		stbuf->st_nlink = 1;
		stbuf->st_size = strlen(hello_str);
	} else
		res = -ENOENT;

	return res;
}

int FuseThread::readdir(const char *path, void *buf, fuse_fill_dir_t filler, off_t offset, struct fuse_file_info *fi)
{
	(void) offset;
	(void) fi;

	if (strcmp(path, "/") != 0)
		return -ENOENT;

	filler(buf, ".", NULL, 0);
	filler(buf, "..", NULL, 0);
	filler(buf, hello_path + 1, NULL, 0);

	return 0;
}

int FuseThread::open(const char *path, struct fuse_file_info *fi)
{
	if (strcmp(path, hello_path) != 0)
		return -ENOENT;

	if ((fi->flags & 3) != O_RDONLY)
		return -EACCES;

	return 0;
}

int FuseThread::read(const char *path, char *buf, size_t size, off_t offset, struct fuse_file_info *fi)
{
	size_t len;
	(void) fi;
	if(strcmp(path, hello_path) != 0)
		return -ENOENT;

	len = strlen(hello_str);
	if (offset < len) {
		if (offset + size > len)
			size = len - offset;
		memcpy(buf, hello_str + offset, size);
	} else
		size = 0;

	return size;
}
